﻿using System;
using System.ComponentModel;
using System.Threading;
using System.Windows;

using Microsoft.Kinect;
using System.Windows.Media;
using System.Windows.Media.Imaging;

namespace WPFColorPolling
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        BackgroundWorker worker = null;

        KinectSensor sensor = null;

        ColorFrameSource colorFrameSource = null;
        ColorFrameReader colorFrameReader = null;
        ColorFrame colorFrame = null;

        public MainWindow()
        {
            InitializeComponent();
        }

        private void Window_Loaded(object sender, RoutedEventArgs e)
        {
            sensor = KinectSensor.GetDefault();

            colorFrameSource = sensor.ColorFrameSource;
            colorFrameReader = colorFrameSource.OpenReader();

            sensor.Open();

            // Create a Thread or use a BackgroundWorker

            worker = new BackgroundWorker();
            worker.DoWork += worker_DoWork;
            worker.RunWorkerCompleted += worker_RunWorkerCompleted;
            worker.ProgressChanged += worker_ProgressChanged;


            worker.WorkerReportsProgress = true;
            worker.WorkerSupportsCancellation = true;

            textBlock1.Text = "";

            worker.RunWorkerAsync();
        }

        private void worker_DoWork(object sender, DoWorkEventArgs e)
        {
            while (true)
            {
                // run all background tasks here
                Thread.Sleep(100); // 0.1 seconds

                colorFrame = colorFrameReader.AcquireLatestFrame();                

                worker.ReportProgress(0); // Can pass int percent and object userState
            }
        }

        private void worker_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
        {
            // Update ui as the worker makes progress on his work
            if (colorFrame != null)
            {
                camera.Source = ToBitmap(colorFrame);
                colorFrame.Dispose(); // <-- IMPORTANT - This was needed to make this all work

                displayTime();
            }            
        }

        private void worker_ProgressChanged(object sender, ProgressChangedEventArgs e)
        {
            // Update ui as the worker makes progress on his work
            if (colorFrame != null)
            {
                camera.Source = ToBitmap(colorFrame);
                colorFrame.Dispose(); // <-- IMPORTANT - This was needed to make this all work

                displayTime();
            }           
        }

        private void displayTime()
        {
            int hour = DateTime.Now.ToLocalTime().Hour;
            int minute = DateTime.Now.ToLocalTime().Minute;
            int second = DateTime.Now.ToLocalTime().Second;
            int millisecond = DateTime.Now.ToLocalTime().Millisecond;

            textBlock1.Text = String.Format("{0}:{1}:{2}.{3}", hour, minute, second, millisecond);
        }

        private void Window_Closing(object sender, System.ComponentModel.CancelEventArgs e)
        {
            worker.CancelAsync();

            worker.Dispose();

            sensor.Close();
        }

        public ImageSource ToBitmap(ColorFrame frame)
        {
            int width = frame.FrameDescription.Width;
            int height = frame.FrameDescription.Height;
            PixelFormat format = PixelFormats.Bgr32;

            byte[] pixels = new byte[width * height * ((format.BitsPerPixel + 7) / 8)];

            if (frame.RawColorImageFormat == ColorImageFormat.Bgra)
            {
                frame.CopyRawFrameDataToArray(pixels);
            }
            else
            {
                frame.CopyConvertedFrameDataToArray(pixels, ColorImageFormat.Bgra);
            }

            int stride = width * format.BitsPerPixel / 8;

            return BitmapSource.Create(width, height, 96, 96, format, null, pixels, stride);
        }
    }
}
